//---     
#include "..\UnlimitedParametersEA.mq5"
//---   
#include "Auxiliary.mqh"
#include "InitializeArrays.mqh"
#include "FileFunctions.mqh"
#include "Enums.mqh"
#include "Errors.mqh"
#include "ToString.mqh"
#include "TradeSignals.mqh"
//---  
struct position_properties
  {
   uint              total_deals;      //  
   bool              exists;           //  /  
   string            symbol;           // 
   long              magic;            //  
   string            comment;          // 
   double            swap;             // 
   double            commission;       //    
   double            first_deal_price; //    
   double            price;            //   
   double            current_price;    //          
   double            last_deal_price;  //    
   double            profit;           // / 
   double            volume;           //   
   double            initial_volume;   //   
   double            sl;               // Stop Loss 
   double            tp;               // Take Profit 
   datetime          time;             //   
   ulong             duration;         //    
   long              id;               //  
   ENUM_POSITION_TYPE type;            // T 
  };
//---  
struct symbol_properties
  {
   int               digits;           //      
   int               spread;           //    
   int               stops_level;      //   Stop 
   double            point;            //   
   double            ask;              //  ask
   double            bid;              //  bid
   double            volume_min;       //     
   double            volume_max;       //     
   double            volume_limit;     //          
   double            volume_step;      //       
   double            offset;           //       
   double            up_level;         //    stop level
   double            down_level;       //    stop level
   ENUM_SYMBOL_TRADE_EXECUTION execution_mode; //   
  };
//---     
position_properties  pos;
symbol_properties    symb;
//+------------------------------------------------------------------+
//|                                                      |
//+------------------------------------------------------------------+
void TradingBlock(int symbol_number)
  {
   ENUM_ORDER_TYPE      signal=WRONG_VALUE;                 //    
   string               comment="hello :)";                 //   
   double               tp=0.0;                             // Take Profit
   double               sl=0.0;                             // Stop Loss
   double               lot=0.0;                            //        
   double               position_open_price=0.0;            //   
   ENUM_ORDER_TYPE      order_type=WRONG_VALUE;             //     
   ENUM_POSITION_TYPE   opposite_position_type=WRONG_VALUE; //   
//--- ,   
   pos.exists=PositionSelect(InputSymbols[symbol_number]);
//---  
   signal=GetTradingSignal(symbol_number);
//---   , 
   if(signal==WRONG_VALUE)
      return;
//---   
   GetSymbolProperties(symbol_number,S_ALL);
//---    
   switch(signal)
     {
      //---     BUY
      case ORDER_TYPE_BUY  :
         position_open_price=symb.ask;
         order_type=ORDER_TYPE_BUY;
         opposite_position_type=POSITION_TYPE_SELL;
         break;
         //---     SELL
      case ORDER_TYPE_SELL :
         position_open_price=symb.bid;
         order_type=ORDER_TYPE_SELL;
         opposite_position_type=POSITION_TYPE_BUY;
         break;
     }
//---   Take Profit  Stop Loss
   sl=CalculateStopLoss(symbol_number,order_type);
   tp=CalculateTakeProfit(symbol_number,order_type);
//---   
   if(!pos.exists)
     {
      //---  
      lot=CalculateLot(symbol_number,InputLot[symbol_number]);
      //---  
      OpenPosition(symbol_number,lot,order_type,position_open_price,sl,tp,comment);
     }
//---   
   else
     {
      //---   
      GetPositionProperties(symbol_number,P_TYPE);
      //---        
      if(pos.type==opposite_position_type && InputReverse[symbol_number])
        {
         //---   
         GetPositionProperties(symbol_number,P_VOLUME);
         //---  
         lot=pos.volume+CalculateLot(symbol_number,InputLot[symbol_number]);
         //---  
         OpenPosition(symbol_number,lot,order_type,position_open_price,sl,tp,comment);
         return;
        }
      //---         ,    
      if(!(pos.type==opposite_position_type) && InputVolumeIncrease[symbol_number]>0)
        {
         //---  Stop Loss  
         GetPositionProperties(symbol_number,P_SL);
         //---  Take Profit  
         GetPositionProperties(symbol_number,P_TP);
         //---  
         lot=CalculateLot(symbol_number,InputVolumeIncrease[symbol_number]);
         //---   
         OpenPosition(symbol_number,lot,order_type,position_open_price,pos.sl,pos.tp,comment);
         return;
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                  |
//+------------------------------------------------------------------+
void OpenPosition(int symbol_number,
                  double lot,
                  ENUM_ORDER_TYPE order_type,
                  double price,
                  double sl,
                  double tp,
                  string comment)
  {
//---      
   trade.SetExpertMagicNumber(MagicNumber);
//---     
   trade.SetDeviationInPoints(CorrectValueBySymbolDigits(Deviation));
//---  Instant Execution  Market Execution
//    ***   803 ,  Stop Loss  Take Profit                             ***
//    ***        SYMBOL_TRADE_EXECUTION_MARKET ***
   if(symb.execution_mode==SYMBOL_TRADE_EXECUTION_INSTANT ||
      symb.execution_mode==SYMBOL_TRADE_EXECUTION_MARKET)
     {
      //---    ,    
      if(!trade.PositionOpen(InputSymbols[symbol_number],order_type,lot,price,sl,tp,comment))
         Print("   : ",GetLastError()," - ",ErrorDescription(GetLastError()));
     }
  }
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
double CalculateLot(int symbol_number,double lot)
  {
//---     
   double corrected_lot=0.0;
//---
   GetSymbolProperties(symbol_number,S_VOLUME_MIN);  //    
   GetSymbolProperties(symbol_number,S_VOLUME_MAX);  //    
   GetSymbolProperties(symbol_number,S_VOLUME_STEP); //   / 
//---     
   corrected_lot=MathRound(lot/symb.volume_step)*symb.volume_step;
//---   ,  
   if(corrected_lot<symb.volume_min)
      return(NormalizeDouble(symb.volume_min,2));
//---   ,  
   if(corrected_lot>symb.volume_max)
      return(NormalizeDouble(symb.volume_max,2));
//---
   return(NormalizeDouble(corrected_lot,2));
  } 
//+------------------------------------------------------------------+
//|   Take Profit                                 |
//+------------------------------------------------------------------+
double CalculateTakeProfit(int symbol_number,ENUM_ORDER_TYPE order_type)
  {
//---  Take Profit 
   if(InputTakeProfit[symbol_number]>0)
     {
      //---    Take Profit
      double tp=0.0;
      //---       SELL
      if(order_type==ORDER_TYPE_SELL)
        {
         //---  
         tp=NormalizeDouble(symb.bid-CorrectValueBySymbolDigits(InputTakeProfit[symbol_number]*symb.point),symb.digits);
         //---   ,      stops level
         //        ,   
         return(tp<symb.down_level ? tp : symb.down_level-symb.offset);
        }
      //---       BUY
      if(order_type==ORDER_TYPE_BUY)
        {
         //---  
         tp=NormalizeDouble(symb.ask+CorrectValueBySymbolDigits(InputTakeProfit[symbol_number]*symb.point),symb.digits);
         //---   ,      stops level
         //        ,   
         return(tp>symb.up_level ? tp : symb.up_level+symb.offset);
        }
     }
//---
   return(0.0);
  } 
//+------------------------------------------------------------------+
//|   Stop Loss                                   |
//+------------------------------------------------------------------+
double CalculateStopLoss(int symbol_number,ENUM_ORDER_TYPE order_type)
  {
//---  Stop Loss 
   if(InputStopLoss[symbol_number]>0)
     {
      //---    Stop Loss
      double sl=0.0;
      //---       BUY
      if(order_type==ORDER_TYPE_BUY)
        {
         //  
         sl=NormalizeDouble(symb.ask-CorrectValueBySymbolDigits(InputStopLoss[symbol_number]*symb.point),symb.digits);
         //---   ,      stops level
         //        ,   
         return(sl<symb.down_level ? sl : symb.down_level-symb.offset);
        }
      //---       SELL
      if(order_type==ORDER_TYPE_SELL)
        {
         //---  
         sl=NormalizeDouble(symb.bid+CorrectValueBySymbolDigits(InputStopLoss[symbol_number]*symb.point),symb.digits);
         //---   ,      stops level
         //        ,   
         return(sl>symb.up_level ? sl : symb.up_level+symb.offset);
        }
     }
//---
   return(0.0);
  } 
//+------------------------------------------------------------------+
//|   Trailing Stop                               |
//+------------------------------------------------------------------+
double CalculateTrailingStop(int symbol_number,ENUM_POSITION_TYPE position_type)
  {
//---   
   double    level       =0.0;
   double    buy_point   =low[symbol_number].value[1];  //  Low  Buy
   double    sell_point  =high[symbol_number].value[1]; //  High  Sell
//---     BUY
   if(position_type==POSITION_TYPE_BUY)
     {
      //---      
      level=NormalizeDouble(buy_point-CorrectValueBySymbolDigits(InputStopLoss[symbol_number]*symb.point),symb.digits);
      //---    ,     (stops level), 
      //      ,    
      if(level<symb.down_level)
         return(level);
      //---    ,      bid
      else
        {
         level=NormalizeDouble(symb.bid-CorrectValueBySymbolDigits(InputStopLoss[symbol_number]*symb.point),symb.digits);
         //---     ,    
         //        
         return(level<symb.down_level ? level : symb.down_level-symb.offset);
        }
     }
//---     SELL
   if(position_type==POSITION_TYPE_SELL)
     {
      //     - 
      level=NormalizeDouble(sell_point+CorrectValueBySymbolDigits(InputStopLoss[symbol_number]*symb.point),symb.digits);
      //---    ,     (stops level), 
      //      ,    
      if(level>symb.up_level)
         return(level);
      //---    ,      ask
      else
        {
         level=NormalizeDouble(symb.ask+CorrectValueBySymbolDigits(InputStopLoss[symbol_number]*symb.point),symb.digits);
         //---     ,    
         //        
         return(level>symb.up_level ? level : symb.up_level+symb.offset);
        }
     }
//---
   return(0.0);
  }
//+------------------------------------------------------------------+
//|   Trailing Stop                                   |
//+------------------------------------------------------------------+
void ModifyTrailingStop(int symbol_number)
  {
//---     StopLoss
   if(InputTrailingStop[symbol_number]>0 && InputStopLoss[symbol_number]>0)
     {
      double         new_sl=0.0;       //     Stop loss
      bool           condition=false;  //     
      //---   / 
      pos.exists=PositionSelect(InputSymbols[symbol_number]);
      //---   
      if(pos.exists)
        {
         //---   
         GetSymbolProperties(symbol_number,S_ALL);
         //---   
         GetPositionProperties(symbol_number,P_ALL);
         //---    Stop Loss
         new_sl=CalculateTrailingStop(symbol_number,pos.type);
         //---           Trailing Stop
         switch(pos.type)
           {
            case POSITION_TYPE_BUY  :
               //---     Stop Loss ,
               //         
               condition=new_sl>pos.sl+CorrectValueBySymbolDigits(InputTrailingStop[symbol_number]*symb.point);
               break;
            case POSITION_TYPE_SELL :
               //---     Stop Loss ,
               //         
               condition=new_sl<pos.sl-CorrectValueBySymbolDigits(InputTrailingStop[symbol_number]*symb.point);
               break;
           }
         //---  Stop Loss ,     
         if(pos.sl>0)
           {
            //---      , ..   /, 
            //     ,    
            if(condition)
              {
               if(!trade.PositionModify(InputSymbols[symbol_number],new_sl,pos.tp))
                  Print("   : ",GetLastError()," - ",ErrorDescription(GetLastError()));
              }
           }
         //---  Stop Loss ,    
         if(pos.sl==0)
           {
            if(!trade.PositionModify(InputSymbols[symbol_number],new_sl,pos.tp))
               Print("   : ",GetLastError()," - ",ErrorDescription(GetLastError()));
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                           |
//+------------------------------------------------------------------+
void GetPositionProperties(int symbol_number,ENUM_POSITION_PROPERTIES position_property)
  {
//--- ,   
   pos.exists=PositionSelect(InputSymbols[symbol_number]);
//---   ,   
   if(pos.exists)
     {
      switch(position_property)
        {
         case P_TOTAL_DEALS      :
            pos.time=(datetime)PositionGetInteger(POSITION_TIME);
            pos.total_deals=CurrentPositionTotalDeals(symbol_number);                              break;
         case P_SYMBOL           : pos.symbol=PositionGetString(POSITION_SYMBOL);                  break;
         case P_MAGIC            : pos.magic=PositionGetInteger(POSITION_MAGIC);                   break;
         case P_COMMENT          : pos.comment=PositionGetString(POSITION_COMMENT);                break;
         case P_SWAP             : pos.swap=PositionGetDouble(POSITION_SWAP);                      break;
         case P_COMMISSION       : pos.commission=PositionGetDouble(POSITION_COMMISSION);          break;
         case P_PRICE_FIRST_DEAL :
            pos.time=(datetime)PositionGetInteger(POSITION_TIME);
            pos.first_deal_price=CurrentPositionFirstDealPrice(symbol_number);                     break;
         case P_PRICE_OPEN       : pos.price=PositionGetDouble(POSITION_PRICE_OPEN);               break;
         case P_PRICE_CURRENT    : pos.current_price=PositionGetDouble(POSITION_PRICE_CURRENT);    break;
         case P_PRICE_LAST_DEAL  :
            pos.time=(datetime)PositionGetInteger(POSITION_TIME);
            pos.last_deal_price=CurrentPositionLastDealPrice(symbol_number);                       break;
         case P_PROFIT           : pos.profit=PositionGetDouble(POSITION_PROFIT);                  break;
         case P_VOLUME           : pos.volume=PositionGetDouble(POSITION_VOLUME);                  break;
         case P_INITIAL_VOLUME   :
            pos.time=(datetime)PositionGetInteger(POSITION_TIME);
            pos.initial_volume=CurrentPositionInitialVolume(symbol_number);                        break;
         case P_SL               : pos.sl=PositionGetDouble(POSITION_SL);                          break;
         case P_TP               : pos.tp=PositionGetDouble(POSITION_TP);                          break;
         case P_TIME             : pos.time=(datetime)PositionGetInteger(POSITION_TIME);           break;
         case P_DURATION         : pos.duration=CurrentPositionDuration(SECONDS);                  break;
         case P_ID               : pos.id=PositionGetInteger(POSITION_IDENTIFIER);                 break;
         case P_TYPE             : pos.type=(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE); break;
         case P_ALL              :
            pos.symbol=PositionGetString(POSITION_SYMBOL);
            pos.magic=PositionGetInteger(POSITION_MAGIC);
            pos.comment=PositionGetString(POSITION_COMMENT);
            pos.swap=PositionGetDouble(POSITION_SWAP);
            pos.commission=PositionGetDouble(POSITION_COMMISSION);
            pos.price=PositionGetDouble(POSITION_PRICE_OPEN);
            pos.current_price=PositionGetDouble(POSITION_PRICE_CURRENT);
            pos.profit=PositionGetDouble(POSITION_PROFIT);
            pos.volume=PositionGetDouble(POSITION_VOLUME);
            pos.sl=PositionGetDouble(POSITION_SL);
            pos.tp=PositionGetDouble(POSITION_TP);
            pos.time=(datetime)PositionGetInteger(POSITION_TIME);
            pos.id=PositionGetInteger(POSITION_IDENTIFIER);
            pos.type=(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);
            pos.total_deals=CurrentPositionTotalDeals(symbol_number);
            pos.first_deal_price=CurrentPositionFirstDealPrice(symbol_number);
            pos.last_deal_price=CurrentPositionLastDealPrice(symbol_number);
            pos.initial_volume=CurrentPositionInitialVolume(symbol_number);
            pos.duration=CurrentPositionDuration(SECONDS);                                         break;
         default: Print("      !");                  return;
        }
     }
//---   ,    
   else
      ZeroPositionProperties();
  }
//+------------------------------------------------------------------+ 
//|                                           |
//+------------------------------------------------------------------+
void GetSymbolProperties(int symbol_number,ENUM_SYMBOL_PROPERTIES symbol_property)
  {
   int lot_offset=1; //       stops level
//---
   switch(symbol_property)
     {
      case S_DIGITS         : symb.digits=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_DIGITS);                   break;
      case S_SPREAD         : symb.spread=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_SPREAD);                   break;
      case S_STOPSLEVEL     : symb.stops_level=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_TRADE_STOPS_LEVEL);   break;
      case S_POINT          : symb.point=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_POINT);                           break;
      //---
      case S_ASK            :
         symb.digits=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_DIGITS);
         symb.ask=NormalizeDouble(SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_ASK),symb.digits);                       break;
      case S_BID            :
         symb.digits=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_DIGITS);
         symb.bid=NormalizeDouble(SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_BID),symb.digits);                       break;
         //---
      case S_VOLUME_MIN     : symb.volume_min=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_VOLUME_MIN);                 break;
      case S_VOLUME_MAX     : symb.volume_max=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_VOLUME_MAX);                 break;
      case S_VOLUME_LIMIT   : symb.volume_limit=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_VOLUME_LIMIT);             break;
      case S_VOLUME_STEP    : symb.volume_step=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_VOLUME_STEP);               break;
      //---
      case S_FILTER         :
         symb.digits=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_DIGITS);
         symb.point=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_POINT);
         symb.offset=NormalizeDouble(CorrectValueBySymbolDigits(lot_offset*symb.point),symb.digits);                          break;
         //---
      case S_UP_LEVEL       :
         symb.digits=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_DIGITS);
         symb.stops_level=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_TRADE_STOPS_LEVEL);
         symb.point=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_POINT);
         symb.ask=NormalizeDouble(SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_ASK),symb.digits);
         symb.up_level=NormalizeDouble(symb.ask+symb.stops_level*symb.point,symb.digits);                                     break;
         //---
      case S_DOWN_LEVEL     :
         symb.digits=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_DIGITS);
         symb.stops_level=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_TRADE_STOPS_LEVEL);
         symb.point=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_POINT);
         symb.bid=NormalizeDouble(SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_BID),symb.digits);
         symb.down_level=NormalizeDouble(symb.bid-symb.stops_level*symb.point,symb.digits);                                   break;
         //---
      case S_EXECUTION_MODE :
         symb.execution_mode=(ENUM_SYMBOL_TRADE_EXECUTION)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_TRADE_EXEMODE); break;
         //---
      case S_ALL            :
         symb.digits=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_DIGITS);
         symb.spread=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_SPREAD);
         symb.stops_level=(int)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_TRADE_STOPS_LEVEL);
         symb.point=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_POINT);
         symb.ask=NormalizeDouble(SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_ASK),symb.digits);
         symb.bid=NormalizeDouble(SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_BID),symb.digits);
         symb.volume_min=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_VOLUME_MIN);
         symb.volume_max=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_VOLUME_MAX);
         symb.volume_limit=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_VOLUME_LIMIT);
         symb.volume_step=SymbolInfoDouble(InputSymbols[symbol_number],SYMBOL_VOLUME_STEP);
         symb.offset=NormalizeDouble(CorrectValueBySymbolDigits(lot_offset*symb.point),symb.digits);
         symb.up_level=NormalizeDouble(symb.ask+symb.stops_level*symb.point,symb.digits);
         symb.down_level=NormalizeDouble(symb.bid-symb.stops_level*symb.point,symb.digits);
         symb.execution_mode=(ENUM_SYMBOL_TRADE_EXECUTION)SymbolInfoInteger(InputSymbols[symbol_number],SYMBOL_TRADE_EXEMODE); break;
         //---
      default : Print("      !"); return;
     }
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
void ZeroPositionProperties()
  {
   pos.symbol       ="";
   pos.exists       =false;
   pos.comment      ="";
   pos.magic        =0;
   pos.price        =0.0;
   pos.current_price=0.0;
   pos.sl           =0.0;
   pos.tp           =0.0;
   pos.type         =WRONG_VALUE;
   pos.volume       =0.0;
   pos.commission   =0.0;
   pos.swap         =0.0;
   pos.profit       =0.0;
   pos.time         =NULL;
   pos.id           =0;
  }
//+------------------------------------------------------------------+
//|                          |
//+------------------------------------------------------------------+
uint CurrentPositionTotalDeals(int symbol_number)
  {
   int    total       =0;  //      
   int    count       =0;  //     
   string deal_symbol =""; //  
//---    
   if(HistorySelect(pos.time,TimeCurrent()))
     {
      //---      
      total=HistoryDealsTotal();
      //---       
      for(int i=0; i<total; i++)
        {
         //---   
         deal_symbol=HistoryDealGetString(HistoryDealGetTicket(i),DEAL_SYMBOL);
         //---       ,  
         if(deal_symbol==InputSymbols[symbol_number])
            count++;
        }
     }
//---
   return(count);
  }
//+------------------------------------------------------------------+
//|                          |
//+------------------------------------------------------------------+
double CurrentPositionFirstDealPrice(int symbol_number)
  {
   int      total       =0;    //      
   string   deal_symbol ="";   //  
   double   deal_price  =0.0;  //  
   datetime deal_time   =NULL; //  
//---    
   if(HistorySelect(pos.time,TimeCurrent()))
     {
      //---      
      total=HistoryDealsTotal();
      //---       
      for(int i=0; i<total; i++)
        {
         //---   
         deal_price=HistoryDealGetDouble(HistoryDealGetTicket(i),DEAL_PRICE);
         //---   
         deal_symbol=HistoryDealGetString(HistoryDealGetTicket(i),DEAL_SYMBOL);
         //---   
         deal_time=(datetime)HistoryDealGetInteger(HistoryDealGetTicket(i),DEAL_TIME);
         //---        , 
         //           ,   
         if(deal_time==pos.time && deal_symbol==InputSymbols[symbol_number])
            break;
        }
     }
//---
   return(deal_price);
  }
//+------------------------------------------------------------------+
//|                       |
//+------------------------------------------------------------------+
double CurrentPositionLastDealPrice(int symbol_number)
  {
   int    total       =0;   //      
   string deal_symbol ="";  //   
   double deal_price  =0.0; // 
//---    
   if(HistorySelect(pos.time,TimeCurrent()))
     {
      //---      
      total=HistoryDealsTotal();
      //---              
      for(int i=total-1; i>=0; i--)
        {
         //---   
         deal_price=HistoryDealGetDouble(HistoryDealGetTicket(i),DEAL_PRICE);
         //---   
         deal_symbol=HistoryDealGetString(HistoryDealGetTicket(i),DEAL_SYMBOL);
         //---       ,  
         if(deal_symbol==InputSymbols[symbol_number])
            break;
        }
     }
//---
   return(deal_price);
  }
//+------------------------------------------------------------------+
//|                            |
//+------------------------------------------------------------------+
double CurrentPositionInitialVolume(int symbol_number)
  {
   int             total       =0;           //      
   ulong           ticket      =0;           //  
   ENUM_DEAL_ENTRY deal_entry  =WRONG_VALUE; //   
   bool            inout       =false;       //    
   double          sum_volume  =0.0;         //       
   double          deal_volume =0.0;         //  
   string          deal_symbol ="";          //   
   datetime        deal_time   =NULL;        //   
//---    
   if(HistorySelect(pos.time,TimeCurrent()))
     {
      //---      
      total=HistoryDealsTotal();
      //---              
      for(int i=total-1; i>=0; i--)
        {
         //---         , ...
         if((ticket=HistoryDealGetTicket(i))>0)
           {
            //---   
            deal_volume=HistoryDealGetDouble(ticket,DEAL_VOLUME);
            //---    
            deal_entry=(ENUM_DEAL_ENTRY)HistoryDealGetInteger(ticket,DEAL_ENTRY);
            //---    
            deal_time=(datetime)HistoryDealGetInteger(ticket,DEAL_TIME);
            //---   
            deal_symbol=HistoryDealGetString(ticket,DEAL_SYMBOL);
            //---           ,   
            if(deal_time<=pos.time)
               break;
            //---        ,  
            if(deal_symbol==InputSymbols[symbol_number])
               sum_volume+=deal_volume;
           }
        }
     }
//---     - 
   if(deal_entry==DEAL_ENTRY_INOUT)
     {
      //---    /
      //     ,   
      if(fabs(sum_volume)>0)
        {
         //---        
         double result=pos.volume-sum_volume;
         //---    ,  ,              
         deal_volume=result>0 ? result : pos.volume;
        }
      //---       ,
      if(sum_volume==0)
         deal_volume=pos.volume; //    
     }
//---    
   return(NormalizeDouble(deal_volume,2));
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
ulong CurrentPositionDuration(ENUM_POSITION_DURATION mode)
  {
   ulong     result=0;   //  
   ulong     seconds=0;  //  
//---     
   seconds=TimeCurrent()-pos.time;
//---
   switch(mode)
     {
      case DAYS      : result=seconds/(60*60*24);   break; //  - 
      case HOURS     : result=seconds/(60*60);      break; //  - 
      case MINUTES   : result=seconds/60;           break; //  - 
      case SECONDS   : result=seconds;              break; //   (- )
      //---
      default        :
         Print(__FUNCTION__,"():    !");
         return(0);
     }
//---  
   return(result);
  }
//+------------------------------------------------------------------+
//|                                                |
//+------------------------------------------------------------------+
bool CheckNewBar(int symbol_number)
  {
//---     
//        ,   
   if(CopyTime(InputSymbols[symbol_number],Period(),0,1,lastbar_time[symbol_number].time)==-1)
      Print(__FUNCTION__,":     : "+IntegerToString(GetLastError()));
//---     
   if(new_bar[symbol_number]==NULL)
     {
      //---  
      new_bar[symbol_number]=lastbar_time[symbol_number].time[0];
      Print(__FUNCTION__,":  ["+InputSymbols[symbol_number]+"][TF: "+TimeframeToString(Period())+"]["
            +TimeToString(lastbar_time[symbol_number].time[0],TIME_DATE|TIME_MINUTES|TIME_SECONDS)+"]");
      return(false);
     }
//---   
   if(new_bar[symbol_number]!=lastbar_time[symbol_number].time[0])
     {
      //---    
      new_bar[symbol_number]=lastbar_time[symbol_number].time[0];
      return(true);
     }
//---     -    ,  false
   return(false);
  }
//+------------------------------------------------------------------+
